package e2e_test

// type applyMachineOS struct {
// 	restart bool

// 	cmd []string
// }

// func (a *applyMachineOS) buildCmd(m *machineTestBuilder) []string {
// 	cmd := []string{"machine", "os", "apply"}
// 	if a.restart {
// 		cmd = append(cmd, "--restart")
// 	}

// 	a.cmd = cmd
// 	return cmd
// }

// func (a *applyMachineOS) withRestart() *applyMachineOS {
// 	a.restart = true
// 	return a
// }

// func (a *applyMachineOS) args(cmd []string) *applyMachineOS {
// 	a.cmd = cmd
// 	return a
// }
